@@ -23,7 +23,6 @@ if False:
 if sys.platform.startswith("darwin"): 
     #crashes were reported with the Intel driver on OSX 
     BLACKLIST["vendor"].append("Intel Inc.")
-    WHITELIST["renderer"] = ["Intel HD Graphics 4000 OpenGL Engine"]
 
 
 DEFAULT_ALPHA = not sys.platform.startswith("win") and not sys.platform.startswith("darwin")
@@ -98,6 +97,31 @@ def raise_error(msg):
 gl_check_error = raise_error
 
 
+#support for memory views requires Python 2.7 and PyOpenGL 3.1
+def is_pyopengl_memoryview_safe(pyopengl_version):
+    if sys.version_info[:2]<(2,7):
+        #memoryview type requires python 2.7
+        return False
+    vsplit = pyopengl_version.split('.')
+    if vsplit[:2]<['3','1']:
+        #requires PyOpenGL >= 3.1, earlier versions will not work
+        return False
+    if vsplit[:2]>=['3','2']:
+        #assume that newer versions are OK too
+        return True
+    #at this point, we know we have a 3.1.x version, but which one?
+    if len(vsplit)<3:
+        #not enough parts to know for sure, assume it's not supported
+        return False
+    micro = vsplit[2]
+    #ie: '0', '1' or '0b2'
+    if micro=='0':
+        return True     #3.1.0 is OK
+    if micro>='1':
+        return True     #3.1.1 onwards should be too
+    return False        #probably something like '0b2' which is broken
+
+
 def check_functions(*functions):
     missing = []
     available = []
@@ -153,9 +177,18 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
                               (".".join([str(x) for x in MIN_VERSION]), gl_major, gl_minor))
         else:
             log("found valid OpenGL version: %s.%s", gl_major, gl_minor)
+
+	from OpenGL import version as OpenGL_version
+        try:
+            import OpenGL_accelerate
+        except:
+            OpenGL_accelerate = None
+        props["zerocopy"] = is_pyopengl_memoryview_safe(OpenGL_version.__version__) and bool(OpenGL_accelerate)
+
         try:
             extensions = glGetString(GL_EXTENSIONS).split(" ")
         except:
+            extensions = []
             gl_check_error("OpenGL could not find the list of GL extensions - does the graphics driver support OpenGL?")
         log("OpenGL extensions found: %s", ", ".join(extensions))
         props["extensions"] = extensions
@@ -423,9 +456,11 @@ def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
 
 def main():
     from xpra.platform import init,clean
+    from xpra.platform.gui import init as gui_init
     from xpra.util import pver
     try:
         init("OpenGL-Check")
+        gui_init()
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
         if verbose:
             log.enable_debug()
