@@ -6,11 +6,13 @@
 
 import sys
 import os
-from xpra.gtk_common.gobject_compat import import_gtk, import_gobject
+from xpra.gtk_common.gobject_compat import import_gtk, import_gobject, is_gtk3
 gtk = import_gtk()
 gobject = import_gobject()
 
-from xpra.gtk_common.gtk_util import set_tooltip_text, CheckMenuItem, ensure_item_selected, set_checkeditems, menuitem
+from xpra.util import CLIENT_EXIT
+from xpra.gtk_common.gtk_util import set_tooltip_text, ensure_item_selected, menuitem, BUTTON_PRESS_MASK
+from xpra.client.client_base import EXIT_OK
 from xpra.client.gtk_base.about import about, close_about
 from xpra.codecs.loader import PREFERED_ENCODING_ORDER, ENCODINGS_HELP, ENCODINGS_TO_NAME
 from xpra.log import Logger
@@ -47,6 +49,26 @@ SPEED_OPTIONS[1]    = "Lowest Bandwidth"
 SPEED_OPTIONS[100]  = "Lowest Latency"
 
 
+class TrayCheckMenuItem(gtk.CheckMenuItem):
+    """ We add a button handler to catch clicks that somehow do not
+        trigger the "toggled" signal on some platforms (win32?) when we
+        show the tray menu with a right click and click on the item with the left click.
+        (or the other way around?)
+    """
+    def __init__(self, label, tooltip=None):
+        gtk.CheckMenuItem.__init__(self, label)
+        self.label = label
+        if tooltip:
+            self.set_tooltip_text(tooltip)
+        self.add_events(BUTTON_PRESS_MASK)
+        self.connect("button-release-event", self.on_button_release_event)
+
+    def on_button_release_event(self, *args):
+        log("TrayCheckMenuItem.on_button_release_event(%s) label=%s", args, self.label)
+        self.set_active(True)
+        self.toggled()
+
+
 def make_min_auto_menu(title, min_options, options, get_current_min_value, get_current_value, set_min_value_cb, set_value_cb):
     #note: we must keep references to the parameters on the submenu
     #(closures and gtk callbacks don't mix so well!)
@@ -67,7 +89,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
             options[value] = "%s%%" % value
         for s in sorted(options.keys()):
             t = options.get(s)
-            qi = CheckMenuItem(t)
+            qi = TrayCheckMenuItem(t)
             qi.set_draw_as_radio(True)
             candidate_match = s>=max(0, value)
             qi.set_active(not found_match and candidate_match)
@@ -151,16 +173,15 @@ def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding
         name = ENCODINGS_TO_NAME.get(encoding, encoding)
         descr = ENCODINGS_HELP.get(encoding)
         NAME_TO_ENCODING[name] = encoding
-        encoding_item = CheckMenuItem(name)
+        encoding_item = TrayCheckMenuItem(name)
         if descr:
             if encoding not in server_encodings:
                 descr += "\n(not available on this server)"
             set_tooltip_text(encoding_item, descr)
-        def encoding_changed(oitem):
-            log("encoding_changed(%s)", oitem)
-            item = ensure_item_selected(encodings_submenu, oitem)
+        def encoding_changed(item):
+            ensure_item_selected(encodings_submenu, item)
             enc = NAME_TO_ENCODING.get(item.get_label())
-            log("encoding_changed(%s) item=%s, enc=%s, current=%s", oitem, item, enc, encodings_submenu.get_current_encoding())
+            log("encoding_changed(%s) enc=%s, current=%s", item, enc, encodings_submenu.get_current_encoding())
             if enc is not None and encodings_submenu.get_current_encoding()!=enc:
                 encodings_submenu.set_encoding(enc)
         log("make_encodingsmenu(..) encoding=%s, current=%s, active=%s", encoding, get_current_encoding(), encoding==get_current_encoding())
@@ -192,6 +213,10 @@ class GTKTrayMenuBase(object):
     def show_session_info(self, *args):
         self.client.show_session_info(*args)
 
+    def show_bug_report(self, *args):
+        self.client.show_bug_report(*args)
+
+
     def get_image(self, *args):
         return self.client.get_image(*args)
 
@@ -206,6 +231,7 @@ class GTKTrayMenuBase(object):
 
         menu.append(self.make_aboutmenuitem())
         menu.append(self.make_sessioninfomenuitem())
+        menu.append(self.make_bugreportmenuitem())
         menu.append(gtk.SeparatorMenuItem())
         menu.append(self.make_bellmenuitem())
         if self.client.windows_enabled:
@@ -284,9 +310,10 @@ class GTKTrayMenuBase(object):
             image = self.get_image(icon_name, 24)
         return menuitem(title, image, tooltip, cb)
 
-    def checkitem(self, title, cb=None):
-        """ Utility method for easily creating a CheckMenuItem """
-        check_item = CheckMenuItem(title)
+    def checkitem(self, title, cb=None, active=False):
+        """ Utility method for easily creating a TrayCheckMenuItem """
+        check_item = TrayCheckMenuItem(title)
+        check_item.set_active(active)
         if cb:
             check_item.connect("toggled", cb)
         check_item.show()
@@ -307,6 +334,12 @@ class GTKTrayMenuBase(object):
             self.show_session_info()
         return  self.handshake_menuitem(title, "statistics.png", None, show_session_info_cb)
 
+    def make_bugreportmenuitem(self):
+        def show_bug_report_cb(*args):
+            self.show_bug_report()
+        return  self.handshake_menuitem("Bug Report", "bugs.png", None, show_bug_report_cb)
+
+
     def make_bellmenuitem(self):
         def bell_toggled(*args):
             v = self.bell_menuitem.get_active()
@@ -318,6 +351,7 @@ class GTKTrayMenuBase(object):
         self.bell_menuitem = self.checkitem("Bell", bell_toggled)
         self.bell_menuitem.set_sensitive(False)
         def set_bell_menuitem(*args):
+            log("set_bell_menuitem%s enabled=%s", args, self.client.bell_enabled)
             self.bell_menuitem.set_active(self.client.bell_enabled)
             c = self.client
             can_toggle_bell = c.toggle_cursors_bell_notify and c.server_supports_bell and c.client_supports_bell
@@ -342,6 +376,7 @@ class GTKTrayMenuBase(object):
         self.cursors_menuitem = self.checkitem("Cursors", cursors_toggled)
         self.cursors_menuitem.set_sensitive(False)
         def set_cursors_menuitem(*args):
+            log("set_cursors_menuitem%s enabled=%s", args, self.client.cursors_enabled)
             self.cursors_menuitem.set_active(self.client.cursors_enabled)
             c = self.client
             can_toggle_cursors = c.toggle_cursors_bell_notify and c.server_supports_cursors and c.client_supports_cursors
@@ -358,12 +393,13 @@ class GTKTrayMenuBase(object):
             v = self.notifications_menuitem.get_active()
             changed = self.client.notifications_enabled != v
             self.client.notifications_enabled = v
+            log("notifications_toggled%s active=%s changed=%s", args, v, changed)
             if changed and self.client.toggle_cursors_bell_notify:
                 self.client.send_notify_enabled()
-            log("notifications_toggled(%s) notifications_enabled=%s", args, self.client.notifications_enabled)
         self.notifications_menuitem = self.checkitem("Notifications", notifications_toggled)
         self.notifications_menuitem.set_sensitive(False)
         def set_notifications_menuitem(*args):
+            log("set_notifications_menuitem%s enabled=%s", args, self.client.notifications_enabled)
             self.notifications_menuitem.set_active(self.client.notifications_enabled)
             c = self.client
             can_notify = c.toggle_cursors_bell_notify and c.server_supports_notifications and c.client_supports_notifications
@@ -385,6 +421,7 @@ class GTKTrayMenuBase(object):
         self.clipboard_menuitem = self.checkitem("Clipboard", menu_clipboard_toggled)
         self.clipboard_menuitem.set_sensitive(False)
         def set_clipboard_menuitem(*args):
+            log("set_clipboard_menuitem%s enabled=%s", args, self.client.clipboard_enabled)
             self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
             c = self.client
             can_clipboard = c.server_supports_clipboard and c.client_supports_clipboard
@@ -418,10 +455,10 @@ class GTKTrayMenuBase(object):
                             "Secondary" : "SECONDARY"}
             from xpra.clipboard.translated_clipboard import TranslatedClipboardProtocolHelper
             for label, remote_clipboard in LABEL_TO_NAME.items():
-                clipboard_item = CheckMenuItem(label)
+                clipboard_item = TrayCheckMenuItem(label)
                 def remote_clipboard_changed(item):
                     assert can_clipboard
-                    item = ensure_item_selected(clipboard_submenu, item)
+                    ensure_item_selected(clipboard_submenu, item)
                     label = item.get_label()
                     remote_clipboard = LABEL_TO_NAME.get(label)
                     old_state = self.client.clipboard_enabled
@@ -486,6 +523,7 @@ class GTKTrayMenuBase(object):
         self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
         self.keyboard_sync_menuitem.set_sensitive(False)
         def set_keyboard_sync_menuitem(*args):
+            log("set_keyboard_sync_menuitem%s enabled=%s", args, self.client.keyboard_helper.keyboard_sync)
             self.keyboard_sync_menuitem.set_active(self.client.keyboard_helper.keyboard_sync)
             self.keyboard_sync_menuitem.set_sensitive(self.client.toggle_keyboard_sync)
             set_keyboard_sync_tooltip()
@@ -502,6 +540,7 @@ class GTKTrayMenuBase(object):
                 set_tooltip_text(gl, "no server support for runtime switching")
                 return
             def opengl_toggled(*args):
+                log("opengl_toggled%s", args)
                 self.client.toggle_opengl()
             gl.connect("toggled", opengl_toggled)
         self.client.connect("handshake-complete", gl_set)
@@ -511,6 +550,7 @@ class GTKTrayMenuBase(object):
         encodings = self.menuitem("Encoding", "encoding.png", "Choose picture data encoding", None)
         encodings.set_sensitive(False)
         def set_encodingsmenuitem(*args):
+            log("set_encodingsmenuitem%s", args)
             encodings.set_sensitive(not self.client.mmap_enabled)
             if self.client.mmap_enabled:
                 #mmap disables encoding and uses raw rgb24
@@ -522,7 +562,11 @@ class GTKTrayMenuBase(object):
         return encodings
 
     def make_encodingssubmenu(self, handshake_complete=True):
-        encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
+        all_encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
+        encodings = [x for x in all_encodings if x not in self.client.server_encodings_problematic]
+        if not encodings:
+            #all we have, show the "bad" hidden ones then!
+            encodings = all_encodings
         encodings_submenu = make_encodingsmenu(self.get_current_encoding, self.set_current_encoding, encodings, self.client.server_encodings)
         self.popup_menu_workaround(encodings_submenu)
         return encodings_submenu
@@ -557,7 +601,6 @@ class GTKTrayMenuBase(object):
     def make_qualitysubmenu(self):
         quality_submenu = make_min_auto_menu("Quality", MIN_QUALITY_OPTIONS, QUALITY_OPTIONS,
                                            self.get_min_quality, self.get_quality, self.set_min_quality, self.set_quality)
-        #WARNING: this changes "min-quality", not "quality" (or at least it tries to..)
         self.popup_menu_workaround(quality_submenu)
         quality_submenu.show_all()
         return quality_submenu
@@ -683,7 +726,7 @@ class GTKTrayMenuBase(object):
         menu = gtk.Menu()
         menu.ignore_events = False
         def onoffitem(label, active, cb):
-            c = CheckMenuItem(label)
+            c = TrayCheckMenuItem(label)
             c.set_draw_as_radio(True)
             c.set_active(active)
             def submenu_uncheck(item, menu):
@@ -726,18 +769,28 @@ class GTKTrayMenuBase(object):
         self.layout_submenu = gtk.Menu()
         keyboard.set_submenu(self.layout_submenu)
         self.popup_menu_workaround(self.layout_submenu)
-        def kbitem(title, layout, variant):
+        def kbitem(title, layout, variant, active=False):
             def set_layout(item):
                 """ this callback updates the client (and server) if needed """
-                item = ensure_item_selected(self.layout_submenu, item)
+                ensure_item_selected(self.layout_submenu, item)
                 layout = item.keyboard_layout
                 variant = item.keyboard_variant
-                if layout!=self.client.xkbmap_layout or variant!=self.client.xkbmap_variant:
-                    log("keyboard layout selected: %s / %s", layout, variant)
-                    self.client.xkbmap_layout = layout
-                    self.client.xkbmap_variant = variant
-                    self.client.send_layout()
-            l = self.checkitem(title, set_layout)
+                kh = self.client.keyboard_helper
+                kh.locked = layout!="Auto"
+                if layout!=kh.xkbmap_layout or variant!=kh.xkbmap_variant:
+                    if layout=="Auto":
+                        #re-detect everything:
+                        kh.update()
+                        log.info("keyboard automatic mode: %s", kh.layout_str())
+                        kh.send_layout()
+                        kh.send_keymap()
+                    else:
+                        #use layout specified and send it:
+                        kh.xkbmap_layout = layout
+                        kh.xkbmap_variant = variant
+                        log.info("new keyboard layout selected: %s", kh.layout_str())
+                        kh.send_layout()
+            l = self.checkitem(title, set_layout, active)
             l.set_draw_as_radio(True)
             l.keyboard_layout = layout
             l.keyboard_variant = variant
@@ -745,13 +798,28 @@ class GTKTrayMenuBase(object):
         def keysort(key):
             c,l = key
             return c.lower()+l.lower()
-        layout,variant,variants = self.client.keyboard_helper.keyboard.get_layout_spec()
-        if layout and len(variants)>1:
+        layout,layouts,variant,variants = self.client.keyboard_helper.keyboard.get_layout_spec()
+        full_layout_list = False
+        if len(layouts)>1:
+            log.info("keyboard layouts: %s", ",".join(layouts))
+            auto = kbitem("Auto", "Auto", "", True)
+            self.layout_submenu.append(auto)
+            if layout:
+                self.layout_submenu.append(kbitem("%s" % layout, layout, ""))
+            if variants:
+                for v in variants:
+                    self.layout_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
+            for l in layouts:
+                if l!=layout:
+                    self.layout_submenu.append(kbitem("%s" % l, l, ""))
+        elif layout and variants and len(variants)>1:
             #just show all the variants to choose from this layout
-            self.layout_submenu.append(kbitem("%s - Default" % layout, layout, None))
+            default = kbitem("%s - Default" % layout, layout, "", True)
+            self.layout_submenu.append(default)
             for v in variants:
                 self.layout_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
         else:
+            full_layout_list = True
             from xpra.keyboard.layouts import X11_LAYOUTS
             #show all options to choose from:
             sorted_keys = list(X11_LAYOUTS.keys())
@@ -774,19 +842,14 @@ class GTKTrayMenuBase(object):
                     #no variants:
                     self.layout_submenu.append(kbitem(name, layout, None))
         keyboard_helper = self.client.keyboard_helper
-        def set_selected_layout(*args):
-            if keyboard_helper.xkbmap_layout or keyboard_helper.xkbmap_print or keyboard_helper.xkbmap_query:
+        def set_layout_enabled(*args):
+            if full_layout_list and (keyboard_helper.xkbmap_layout or keyboard_helper.xkbmap_print or keyboard_helper.xkbmap_query):
                 #we have detected a layout
-                #so no need to let the user override it
+                #so no need to show the user the huge layout list
                 keyboard.hide()
                 return
             keyboard.set_sensitive(True)
-            layout = keyboard_helper.xkbmap_layout
-            variant = keyboard_helper.xkbmap_variant
-            def is_match(checkitem):
-                return checkitem.keyboard_layout==layout and checkitem.keyboard_variant==variant
-            set_checkeditems(self.layout_submenu, is_match)
-        self.client.connect("handshake-complete", set_selected_layout)
+        self.client.connect("handshake-complete", set_layout_enabled)
         return keyboard
 
     def make_compressionmenu(self):
@@ -797,13 +860,13 @@ class GTKTrayMenuBase(object):
         self.popup_menu_workaround(self.compression_submenu)
         compression_options = {0 : "None"}
         def set_compression(item):
-            item = ensure_item_selected(self.compression_submenu, item)
+            ensure_item_selected(self.compression_submenu, item)
             c = int(item.get_label().replace("None", "0"))
             if c!=self.client.compression_level:
                 log("setting compression level to %s", c)
                 self.client.set_deflate_level(c)
         for i in range(0, 10):
-            c = CheckMenuItem(str(compression_options.get(i, i)))
+            c = TrayCheckMenuItem(str(compression_options.get(i, i)))
             c.set_draw_as_radio(True)
             c.set_active(i==self.client.compression_level)
             c.connect('activate', set_compression)
@@ -830,7 +893,7 @@ class GTKTrayMenuBase(object):
 
     def make_disconnectmenuitem(self):
         def menu_quit(*args):
-            self.client.quit(0)
+            self.client.disconnect_and_quit(EXIT_OK, CLIENT_EXIT)
         return self.handshake_menuitem("Disconnect", "quit.png", None, menu_quit)
 
     def make_closemenuitem(self):
@@ -839,7 +902,7 @@ class GTKTrayMenuBase(object):
 
     def popup_menu_workaround(self, menu):
         #win32 workaround:
-        if sys.platform.startswith("win"):
+        if sys.platform.startswith("win") and not is_gtk3():
             self.add_popup_menu_workaround(menu)
 
     def add_popup_menu_workaround(self, menu):
