@@ -255,11 +255,7 @@ class WindowVideoSource(WindowSource):
         self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if x in nv_common]
         log("do_set_client_properties(%s) csc_modes=%s, full_csc_modes=%s, video_scaling=%s, video_subregion=%s, uses_swscale=%s, non_video_encodings=%s", properties, self.csc_modes, self.full_csc_modes, self.supports_video_scaling, self.supports_video_subregion, self.uses_swscale, self.non_video_encodings)
 
-    def get_best_encoding_impl(self):
-        #if we're here, the window does not have any alpha (or the client does not support it)
-        #and it is a normal window (not a POPUP or other)
-        if self.strict or self.encoding not in self.video_encodings:
-            return WindowSource.get_best_encoding_impl(self)
+    def get_best_encoding_impl_default(self):
         return self.get_best_encoding_video
 
 
@@ -279,6 +275,9 @@ class WindowVideoSource(WindowSource):
             q = 100
             return self.get_best_nonvideo_encoding(pixel_count, ww, wh, s, q, self.non_video_encodings[0], self.non_video_encodings)
 
+        if len(self.non_video_encodings)==0:
+            return current_encoding
+
         #if speed is high, assume we have bandwidth to spare
         if pixel_count<=self._rgb_auto_threshold:
             return lossless("low pixel count")
